module declar_toy_retreat

use grisli_3D_toy
use declar_io_ncdf_toy

implicit none

! utilise dans les calculs specifiques au recul force

integer,parameter            :: nb_regions = 14   ! nombre de regions
integer                      :: ijob              !< number of the job

character(len=100) :: number_data_B_file          ! nombre de donnees socle dans la maille
character(len=100) :: region_file                 ! regions pour les temps de demarrage
character(len=100) :: file_exp                    ! nom du fichier experiences  
real               :: retreat_1                   !< taux de retrait 1 si beta < beta_lim en km
real               :: retreat_2                   !< taux de retrait 2 si beta > beta_lim 
real               :: beta_lim_ret                !< limite valeur beta
real               :: alpha_lim_1                 !< limite mini de alpha (alpha < alpha_lim_1 -> retreat_1)
real               :: alpha_lim_2                 !< limite maxi de alpha (alpha > alpha_lim_2 -> retreat_2)
                                                  !< lineaire en alpha entre les deux. alpha = log10 (beta)
real               :: bed_sigma_1                 !< pour le sigma du socle quand il y a des donnees
real               :: bed_sigma_2                 !< pour le sigma du socle quand il n'y a pas de donnees


integer, dimension(nx,ny)          :: map_region  !< regions
real, dimension(nx,ny)             :: Bsoc_sigma  !< incertitude sur Bsoc : en entree nombre de points de mesure
real, dimension(nx,ny)             :: B_test      !< pour le test sur la pente du socle
real, dimension(nx,ny)             :: B_voisin    !< pour le test sur la pente du socle

real,dimension(nx,ny)              :: delHdx_mx   !< variation d'epaisseur imposee venant de x
real,dimension(nx,ny)              :: delHdy_my   !< variation d'epaisseur imposee venant de y
real,dimension(nx,ny)              :: delHdx_SW   !< variation d'epaisseur imposee venant de diag SW-NE
real,dimension(nx,ny)              :: delHdx_SE   !< variation d'epaisseur imposee venant de diag SE-NW (noeuds \)

real,dimension(nx,ny)              :: delHdt      !< variation d'epaisseur totale
real,dimension(nx,ny)              :: delHdt_sanity !< variation d'epaisseur maxi autorise par le sanity check
real,dimension(nx,ny)              :: H_float     !< epaisseur de flottaison
real,dimension(nx,ny)              :: H_float_test !< epaisseur de flottaison pour le socle test
real,dimension(nx,ny)              :: time_float  !< temps auquel le noeud se met a flotter
real,dimension(nx,ny)              :: coef_schoof !< coefficient du flux de schoof
real,dimension(nx,ny)              :: ux_schoof   !< vitesse schoof sur les noeuds majeurs (fonction de Hfloat)
real,dimension(nx,ny)              :: uy_schoof   !< vitesse schoof sur les noeuds majeurs (fonction de Hfloat)
real,dimension(nx,ny)              :: bf_x_schoof ! rapport des contraintes longitudinales /unconfined
real,dimension(nx,ny)              :: bf_y_schoof ! rapport des contraintes longitudinales /unconfined
real,dimension(nx,ny)              :: tauf_schoof !< contrainte non confinee

! modulation des temps de depart en fonction de l'ocean

real,dimension(0:nb_regions)       :: time_region !< temps a partir duquel le recul est autorise par region
                                                  !< la region 0 est l'ocean
real,dimension(nx,ny)              :: time_dep    !< temps a partir duquel le recul est autorise (noeuds centres)
real,dimension(nx,ny)              :: time_dep_mx !< temps a partir duquel le recul est autorise (noeuds >)
real,dimension(nx,ny)              :: time_dep_my !< temps a partir duquel le recul est autorise (noeuds ^)
real,dimension(nx,ny)              :: time_dep_SW !< temps a partir duquel le recul est autorise (noeuds /)
real,dimension(nx,ny)              :: time_dep_SE !< temps a partir duquel le recul est autorise (noeuds \)


!< position grounding line sur les mailles mineures 

real,dimension(nx,ny)              :: x_gl_mx     !< position sous maille de la grounding line selon x
real,dimension(nx,ny)              :: y_gl_my     !< position sous maille de la grounding line selon y
real,dimension(nx,ny)              :: SW_gl_m     !< position sous maille de la grounding line selon SW-NE (noeuds /) 
real,dimension(nx,ny)              :: SE_gl_m     !< position sous maille de la grounding line selon SE-NW (noeuds \)
                            
integer,dimension(nx,ny)           :: mk_traiter  !< masque des noeuds a traiter
integer,dimension(nx,ny)           :: mk_traiter0 !< masque des noeuds a traiter initial
integer,dimension(nx,ny)           :: mk_gr       !< masque des noeuds flottants

! taux de retrait initial (fonction du beta)
real,dimension(nx,ny)              :: retreat0_x  !< vitesse maxi autorisee selon x : valeur initiale
real,dimension(nx,ny)              :: retreat0_y  !< vitesse maxi autorisee selon y : valeur initiale
real,dimension(nx,ny)              :: retreat0_SW !< vitesse maxi autorisee / SW-NE : valeur initiale  (noeuds /)
real,dimension(nx,ny)              :: retreat0_SE !< vitesse maxi autorisee \ SE-NW : valeur initiale  (noeuds \)

! taux de retrait  fonction du temps
real,dimension(nx,ny)              :: retreat_x   !< vitesse maxi autorisee selon x : module par le temps
real,dimension(nx,ny)              :: retreat_y   !< vitesse maxi autorisee selon x : module par le temps
real,dimension(nx,ny)              :: retreat_SW  !< vitesse maxi autorisee / SW-NE : valeur initiale  (noeuds /)
real,dimension(nx,ny)              :: retreat_SE  !< vitesse maxi autorisee \ SE-NW : module par le temps  (noeuds \)



! tableaux de travail toutes directions
real,dimension(nx,ny)              :: travail_centre !< valeur sur les noeuds centres
real,dimension(nx,ny)              :: travail_mx     !< valeur sur noeuds >
real,dimension(nx,ny)              :: travail_my     !< valeur sur noeuds ^
real,dimension(nx,ny)              :: travail_SW     !< valeur sur noeuds /
real,dimension(nx,ny)              :: travail_SE     !< valeur sur noeuds \


real,dimension(nx,ny)              :: epsmax         !< taux maxi de deformation

! pour l'implementation 
real                               :: cst_schoof   ! constante qui entre dans le calcul du flux de Schoof
real                               :: nschoof      ! exposant de la loi de deformation pour Schoof
real                               :: mschoof      ! exposant de la loi de glissement pour Schoof
real                               :: m1_schoof    ! 1/(mschoof+1)
real                               :: exp_schoof   ! exposant de Hg dans le flux de Schoof.

end module declar_toy_retreat
