!> \file temperature_column.f90
!!
!! Implementation of functions and subroutines for vertical temperature calculation 
!!
!<


!///////////////////////////////////////////////////////////////////////
!///////////////////////////////////////////////////////////////////////

!> SUBROUTINE: Temp_column 
!! 
!! Vertical temperature calculation
!!
!! Used modules:
!! - use Icetemp_declar
!! - use Tridiagmod
!! - use temperature_type
!! - use autre_pr_temp_type
!! - use step_temp_type
!>

Subroutine  Temp_column(Ii,Jj,Newtempcol_m,Ct_m,Flot_m,H_m,Tbmer_m,&
     Temperature_m,Autre_pr_temp_m,step_temp_m)

  Use Icetemp_declar
  Use Tridiagmod
  use temperature_type
  use autre_pr_temp_type
  use step_temp_type

  
  Implicit None
  integer :: Ii,Jj
  Real,Dimension(:,:,:),         Intent(Out) :: Newtempcol_m   !< Tableau 1d Vert. Des Temperatures En Sortie
  Real,Dimension(:),         Intent(In) :: Ct_m            !< Tableau 1d Vert. Conductivite Thermique
  Logical,                   Intent(In) :: Flot_m          !< Vrai Si Flot_mtant (Test D'Archimede) 'O'
  Real,                      Intent(In) :: H_m             !< Ice Thickness  'O'
  Real,                      Intent(In) :: Tbmer_m         !< Temperature De La Mer A La Base De L'Ice Shelf
  type (Temperature), intent(inout) :: temperature_m 
  type (Autre_pr_temp), intent(inout) :: Autre_pr_temp_m
  type (step_temp),intent(in) :: step_temp_m


  If (H_m.Gt.10.) Then

     ! Conditions Aux Limites À La Base De La Glace
     !________________________________________________

     ! Base Froide 
     If ( ((Autre_pr_temp_m%Ibase(Ii,Jj).Eq.1).Or.(Autre_pr_temp_m%Ibase(Ii,Jj).Eq.4) &
          .Or. ((Autre_pr_temp_m%Ibase(Ii,Jj).Eq.5).And.(Temperature_m%Temperature(Ii,Jj,Nz_m).Lt.Temperature_m%Temp_melting(Ii,Jj,Nz_m)))) &
          .And..Not.Flot_m) Then

        Autre_pr_temp_m%Ibase(Ii,Jj)=1
        Bb(Nz_m)=1.

        If (Ncond.Eq.1) Then ! Avec Socle
           Dzi=H_m*De*Cm/Ct_m(Nz_m)
           Aa(Nz_m)=-Dzm/(Dzm+Dzi)
           Cc(Nz_m)=-Dzi/(Dzm+Dzi)
           Rr(Nz_m)=H_m*De*Autre_pr_temp_m%Phid(Ii,Jj)*Dzm/Ct_m(Nz_m)/(Dzm+Dzi)
        Else                 ! Sans Socle
           Aa(Nz_m)=-1.
           Cc(Nz_m)=0.
           Rr(Nz_m)=-(Autre_pr_temp_m%Ghf(Ii,Jj)-Autre_pr_temp_m%Phid(Ii,Jj))/Ct_m(Nz_m)*H_m*De
        Endif

     Else

        ! Base Temperee Ou Shelf
        ! ----------------------               
        If (Autre_pr_temp_m%Ibase(Ii,Jj).Eq.5)  Autre_pr_temp_m%Ibase(Ii,Jj)=2
        Autre_pr_temp_m%Ibase(Ii,Jj)=Max(Autre_pr_temp_m%Ibase(Ii,Jj),2)
        Aa(Nz_m)=0.
        Bb(Nz_m)=1.
        Cc(Nz_m)=0.

        If (.Not.Flot_m) Then
           Rr(Nz_m)=Temperature_m%Temp_melting(Ii,Jj,Nz_m)
        Else
           Rr(Nz_m)=Tbmer_m
        End If

     Endif


     If (Ncond.Eq.1) Then ! Avec Socle
        Do K=Nz_m+1,NzNzm_m-1
           Rr(K)=Temperature_m%Temperature(Ii,Jj,K)
        End Do
        Call Tridiag (Aa,Bb,Cc,Rr,Hh,NzNzm_m,Ifail1)

     Else                 ! Sans Socle

        Call Tridiag (Aa,Bb,Cc,Rr,Hh,Nz_m,Ifail1)
     Endif


     ! Temperature Dans Le Socle Si Ncond=0, Lineaire Avec Le Gradient Ghf
     If (Ncond.Eq.0) Then
        Do K=Nz_m+1,NzNzm_m
           Hh(K)=Hh(Nz_m)-Dzm*(K-Nz_m)*Autre_pr_temp_m%Ghf(Ii,Jj)/Cm
        End Do
     Endif

     Do K=1,NzNzm_m
        Newtempcol_m(Ii,Jj,K)=Hh(K)
     End Do
     Autre_pr_temp_m%Tbdot(Ii,Jj)=(Newtempcol_m(Ii,Jj,Nz_m)-Temperature_m%Temperature(Ii,Jj,Nz_m))/step_temp_m%Dtt

     !      ------------------------------------------------- Glace Tres Fine (H<10m Ou H=0)
  Else If (H_m.Le.10.) Then

     !       Pour Eviter Des Problemes Lorsque H Est Inferieur A 10 M
     !       Profil Lineaire, Pente=Flux Geothermique
     !       Le Cas Sans Glace Est Traite De La Meme Facon  (Dou=0)

     ! ........................................ Avec Calcul Dans Le Socle
     If (Ncond.Eq.1.And..Not.Flot_m) Then   
        If (H_m.Gt.0.) Then
           ! Gradient Dans Le Socle
           Dou=(Temperature_m%Temperature(Ii,Jj,Nz_m+1)-Temperature_m%Temperature(Ii,Jj,Nz_m))/Dzm*Cm     
           Dou=Dou/Ct_m(Nz_m)*De*H_m

           Tss=Min(0.,Autre_pr_temp_m%Ts(Ii,Jj))
           Do K=1,Nz_m
              Newtempcol_m(Ii,Jj,K)=Tss+Dou*(K-1.)
           End Do

        Else
           ! Pas De Glace
           Tss=Autre_pr_temp_m%Ts(Ii,Jj)
           Do K=1,Nz_m
              Newtempcol_m(Ii,Jj,K)=Tss
           End Do
        End If

        ! Calcul Dans Le Socle Meme S'Il N'Y A Pas De Glace
        Rr(Nz_m)=Newtempcol_m(Ii,Jj,Nz_m)
        Aa(Nz_m)=0
        Bb(Nz_m)=1
        Cc(Nz_m)=0

        Do K=Nz_m+1,NzNzm_m-1
           Rr(K)=Temperature_m%Temperature(Ii,Jj,K)
        End Do

        ! Creation De Nouveaux Tableaux Juste Pour Tridiag      
        Do K=1,Nzm_m+1
           Abis(K)=Aa(Nz_m-1+K)
           Bbis(K)=Bb(Nz_m-1+K)
           Cbis(K)=Cc(Nz_m-1+K)
           Rbis(K)=Rr(Nz_m-1+K)
        End Do

        Call Tridiag (Abis,Bbis,Cbis,Rbis,Hbis,Nzm_m+1,Ifail1)

        Do K=1,Nzm_m+1
           Hh(Nz_m-1+K)=Hbis(K)
        End Do

        ! ........................................ Sans Calcul Dans Le Socle
     Else
        ! Calotte Posee                    
        If ((H_m.Gt.0.).And..Not.Flot_m) Then
           Dou=-Autre_pr_temp_m%Ghf(Ii,Jj)/Ct_m(Nz_m)*De*H_m
           Tss=Min(0.,Autre_pr_temp_m%Ts(Ii,Jj))
           Do K=1,Nz_m
              Newtempcol_m(Ii,Jj,K)=Tss+Dou*(K-1.)
           End Do

           !  Shelf
        Else If ((H_m.Gt.0.).And.Flot_m) Then
           Tss=Min(0.,Autre_pr_temp_m%Ts(Ii,Jj))
           Dou=(Tbmer_m-Tss)*De
           Do K=1,Nz_m
              Newtempcol_m(Ii,Jj,K)=Tss+Dou*(K-1.)
           End Do

           ! Pas De Glace
        Else
           Tss=Autre_pr_temp_m%Ts(Ii,Jj)      
           Do K=1,Nz_m
              Newtempcol_m(Ii,Jj,K)=Tss
           End Do
        End If

        ! Temperature Dans Le Socle, Lineaire Avec Le Gradient Ghf
        If (.Not.Flot_m) Then
           Do K=Nz_m+1,NzNzm_m
              Hh(K)=Newtempcol_m(Ii,Jj,Nz_m)-Dzm*(K-Nz_m)*Autre_pr_temp_m%Ghf(Ii,Jj)/Cm
           End Do
        Else
           Do K=Nz_m+1,NzNzm_m
              Hh(K)=Tbmer_m-Dzm*(K-Nz_m)*Autre_pr_temp_m%Ghf(Ii,Jj)/Cm
           End Do
        End If

     Endif
     ! Fin Du Test Socle


     Do K=Nz_m+1,NzNzm_m
        Newtempcol_m(Ii,Jj,K)=Hh(K)
     End Do

     Autre_pr_temp_m%Tbdot(Ii,Jj)=(Newtempcol_m(Ii,Jj,Nz_m)-Temperature_m%Temperature(Ii,Jj,Nz_m))/step_temp_m%Dtt
     Autre_pr_temp_m%Bmelt(Ii,Jj)=0.
     Autre_pr_temp_m%Ibase(Ii,Jj)=5
     Autre_pr_temp_m%Phid(Ii,Jj)=0.


  End If

End Subroutine Temp_column

!///////////////////////////////////////////////////////////////////////
!///////////////////////////////////////////////////////////////////////
