!> \file advec_icetemp.f90
!! Subroutines for temperature advection in the heat transfer equation
!<

!> \namespace advec_icetemp
!! Subroutines to calculate the temperature advection 
!<

!> SUBROUTINE: Advec_icetemp
!! Implementation of Routines to calculate advection in the heat transfer equation
!! for a column
!! Used modules:
!!    - use Icetemp_declar
!>

Subroutine  Advec_icetemp(Ii,Jj,Prodq_m,Cro_m,Advecx_m,Advecy_m,Advec_m,Ct_m)

  use icetemp_declar 

  Implicit None
    !< Arguments
  Integer,                     Intent(In)  ::  Ii,Jj          !< Indice Selon X Et Y
  Real, Dimension(Nz),         Intent(In)   :: Prodq_m        !< Tableau 1d Vert. heat production
  Real, Dimension(Nz),         Intent(In)   :: Cro_m          !< Tableau 1d Vert. heat capcity
  Real, Dimension(Nz),         Intent(Inout):: Advecx_m       !< Advection selon x
  Real, Dimension(Nz),         Intent(Inout):: Advecy_m       !< Advection selon y
  Real, Dimension(Nz),         Intent(Inout):: Advec_m        !< Advection total
  Real, Dimension(Nz),         Intent(In)   :: Ct_m           !< Tableau 1d Vert.  thermal cond.

 
  Do K=2,Nz-1


     ! On Remplit Deux Tableaux Pour eviter Des Problemes D'Asymetrie
     !         Avection Selon X  (advecx_m en general > 0)
     !         ----------------
     Advecx_m(K) = Iadvec_w(Ii,Jj) * Ux(Ii,Jj,K) *          & ! ux west if upwind
          (T(Ii,Jj,K) - T(Ii-1,Jj,K))  & ! west T gradient
            
          + Iadvec_e(Ii+1,Jj)* Ux(Ii+1,Jj,K)*      & ! ux east if upwind
          (T(Ii+1,Jj,K)-T(Ii,Jj,K))     ! east T gradient

     Advecx_m(K) =  Advecx_m(K) * Dx11                                        !Dx11=1/Dx


     !         Avection Selon Y
     !         ----------------
     Advecy_m(K) =   Iadvec_s(Ii,Jj) * Uy(Ii,Jj,K) *        & ! uy sud if upwind
          (T(Ii,Jj,K) - T(Ii,Jj-1,K))& ! south T gradient
          
          + Iadvec_n(Ii,Jj+1) * Uy(Ii,Jj+1,K) *   & ! uy nord is upwind
          (T(Ii,Jj+1,K)-T(Ii,Jj,K))    ! north T gradient

     Advecy_m(K) =  Advecy_m(K) * Dx11

     Advec_m(K)=Advecx_m(K)+Advecy_m(K)  ! Advection Totale

  End Do

  !     Fin Boucle K, Ecriture Matrice Tridiag Dans La Glace


  !      -----------------------------Cas General (H>10m)
  thick_ice:  If (H(Ii,Jj).Gt.10.) Then

     !        Variables De Calcul Dans La Glace
     ! dou = dtt/dz^2
     Dou=Dtt/Dee/Dee/ H(Ii,Jj) / H(Ii,Jj)  

     ! Dah = dtt/dz
     Dah=Dtt /Dee /H(Ii,Jj)

     ! thermal conductivity at mid point just below the surface                   
     Ct_bas=2.*(Ct_m(1)*Ct_m(2))/(Ct_m(1)+Ct_m(2))           

     ! surface temperature : cannot go above 0 celsius
     T(Ii,Jj,1)=Min(0.,Ts(Ii,Jj))

     ! surface boundary condition
     Aa(1)=0.
     Bb(1)=1.
     Cc(1)=0.
     Rr(1)=T(Ii,Jj,1)

     Do K=2,Nz-1
        Dzz=Dou/Cro_m(K)

        ! Conductivite Au Milieu Des Mailles
        Ct_haut=Ct_bas
        Ct_bas=2.*(Ct_m(K)*Ct_m(K+1))/(Ct_m(K)+Ct_m(K+1))

        ! Advection Verticale Centree
        Aa(K) = -Dzz * Ct_haut - Uzr(Ii,Jj,K) * Dah / 2.  ! lower diag
        Cc(K) = -Dzz * Ct_bas  + Uzr(Ii,Jj,K) * Dah / 2.  ! upper diag
        Bb(K) =  1.+ Dzz * (Ct_haut+Ct_bas)                                ! diag
        
        Rr(K) =  T(Ii,Jj,K) + Prodq_m(K) * Dtt   ! vector
        Rr(K) =  Rr(K)- Dttdx * (Advec_m(K))

     End DO
  End If thick_ice

End Subroutine Advec_icetemp

